use super::api_error;
use super::FallibleApiResponse;
use super::ToAPI;
use mongod::Model;
use mongod::Referencable;
use mongodb::bson::doc;
use rocket::*;
use serde_json::json;

use crate::library::playlist::Playlist;
use crate::library::playlist::Visibility;
use crate::library::user::User;


#[get("/playlists")]
pub async fn playlists_route(u: User) -> FallibleApiResponse {
    let mut playlists = vec![
        "recent".to_string()
    ];

    let own_playlists = Playlist::find(doc! { "owner": u.reference()}, None).await.unwrap();

    for playlist in own_playlists {
        playlists.push(playlist._id);
    }

    Ok(json!(playlists))
}

#[get("/playlist/<id>")]
pub async fn playlist_route(id: &str, u: User) -> FallibleApiResponse {
    if id == "recents" {
        // todo : recently played
    }

    let playlist = Playlist::get(id).await.ok_or_else(|| api_error("No playlist with that ID found"))?;

    if matches!(playlist.visibility, Visibility::Private) && u.username != playlist.owner.get::<User>().await.username {
        return Err(api_error("Forbidden"));
    }

    Ok(playlist.api().await)
}